package com.example.backend.categories.api;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.backend.categories.model.CategorieEntity;
import com.example.backend.categories.repository.CategorieRepository;
import com.example.backend.categories.service.CategorieService;
import com.example.backend.core.configurations.Constants;

import jakarta.validation.Valid;

import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

@Controller
@RequestMapping(CategorieController.URL)

public class CategorieController {

    public static final String URL = Constants.ADMIN_PREFIX + "/categories";
    private static final String CATEGORIE_VIEW = "categories";
    private static final String CATEGORIE_ATTRIBUTE = "categories";
    private static final String CATEGORIE_EDIT_VIEW = "categorie-edit";

    private final CategorieService categorieService;
    public final CategorieRepository categorieRepository;
    private final ModelMapper modelMapper;

    private static String UPLOAD_DIR = System.getProperty("user.dir") + "/uploads";

    public CategorieController(CategorieService categorieService, ModelMapper modelMapper,
            CategorieRepository categorieRepository) {
        this.categorieService = categorieService;
        this.modelMapper = modelMapper;
        this.categorieRepository = categorieRepository;
    }

    private CategorieDTO toDto(CategorieEntity entity) {
        return modelMapper.map(entity, CategorieDTO.class);
    }

    private CategorieEntity toEntity(CategorieDTO categorieDTO) {
        return modelMapper.map(categorieDTO, CategorieEntity.class);
    }

    @GetMapping()
    public String getAll(Model model) {
        model.addAttribute(
                "categories",
                categorieService.getAll().stream()
                        .map(this::toDto)
                        .toList());
        return CATEGORIE_VIEW;
    }

    @GetMapping("/edit/")
    public String create(Model model) {
        model.addAttribute(CATEGORIE_ATTRIBUTE, new CategorieDTO());
        return CATEGORIE_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(@ModelAttribute(name = CATEGORIE_ATTRIBUTE) @Valid CategorieDTO categorieDTO,
            BindingResult bindingResult, RedirectAttributes redirectAttributes,
            @RequestParam("image") MultipartFile imageFile) {
        try {
            if (bindingResult.hasErrors()) {
                return CATEGORIE_EDIT_VIEW;
            }
            CategorieEntity category = toEntity(categorieDTO);
            if (!imageFile.isEmpty()) {
                Path fileNameAndPath = Paths.get(UPLOAD_DIR, imageFile.getOriginalFilename());
                Files.write(fileNameAndPath, imageFile.getBytes());
                category.setImage("/uploads/" + imageFile.getOriginalFilename());
            }
            categorieService.create(category);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return Constants.REDIRECT_VIEW + URL;
    }
}
